library cnpj_cpf;

import 'dart:math';

import 'package:flutter/services.dart';

abstract class CnpjCpf {
  static final _specialChar = '#';

  static _CnpjCpfFormatter formatter({
    bool cnpj = true,
    bool cpf = true,
  }) =>
      _CnpjCpfFormatter(
        cnpj: cnpj,
        cpf: cpf,
      );

  static bool isValid(String value) {
    if (isCnpj(value)) return _isValidCnpj(value);
    if (isCpf(value)) return _isValidCpf(value);
    return false;
  }

  static bool isCnpj(String value) {
    var _workingValue = _onlyNumbers(value);
    return _workingValue.length == 14;
  }

  static bool isCpf(String value) {
    var _workingValue = _onlyNumbers(value);
    return _workingValue.length <= 11;
  }

  static String _onlyNumbers(String value) {
    return value?.replaceAll(RegExp('[^0-9]'), '')?.trim() ?? '';
  }

  static bool _isValidCnpj(String value) {
    var _workingValue = _onlyNumbers(value);
    if (_workingValue.length != 14) return false;
    var _numbers = _workingValue
        .split('')
        .map<int>((e) => int.parse(e))
        .take(12)
        .toList()
        .reversed
        .toList();
    var _digit1 = _getDitigModule11(_numbers, 2, 9);
    _numbers.insert(0, _digit1);
    var _digit2 = _getDitigModule11(_numbers, 2, 9);
    _numbers.insert(0, _digit2);
    return _numbers.reversed.join('') == _workingValue;
  }

  static bool _isValidCpf(String value) {
    var _workingValue = _onlyNumbers(value);
    if (_workingValue.length != 11) return false;
    var _numbers =
        _workingValue.split('').map<int>((e) => int.parse(e)).take(9).toList();
    var _digit1 = _getDitigModule11(_numbers, 10, 2);
    _numbers.add(_digit1);
    var _digit2 = _getDitigModule11(_numbers, 11, 2);
    _numbers.add(_digit2);
    return _numbers.join('') == _workingValue;
  }

  static int _getDitigModule11(
      List<int> numbers, int iniFactor, int endFactor) {
    var _factor = iniFactor;
    var _sum = 0;
    for (var _number in numbers) {
      _sum += _number * _factor;
      if (_factor == endFactor) {
        _factor = iniFactor;
      } else {
        if (iniFactor > endFactor) {
          _factor--;
        } else {
          _factor++;
        }
      }
    }
    var _module = _sum % 11;
    var _digit = 11 - _module;
    return _digit < 10 ? _digit : 0;
  }

  static String randomCnpj({bool withMask = false}) {
    var _numbers = <int>[];
    _numbers.add(Random.secure().nextInt(8) + 1);
    _numbers.add(0);
    _numbers.add(0);
    _numbers.add(0);
    for (var i = 0; i < 8; i++) {
      _numbers.add(Random.secure().nextInt(9));
    }
    _numbers.insert(0, _getDitigModule11(_numbers, 2, 9));
    _numbers.insert(0, _getDitigModule11(_numbers, 2, 9));
    var _cnpj = _numbers.reversed.join();
    if (withMask) {
      return _maskCnpj(_cnpj);
    }
    return _cnpj;
  }

  static String _maskCnpj(String value) {
    var _workingValue = _onlyNumbers(value).padRight(11, _specialChar);
    _workingValue = '${_workingValue.substring(0, 2)}.'
        '${_workingValue.substring(2, 5)}.'
        '${_workingValue.substring(5, 8)}/'
        '${_workingValue.substring(8, 12)}-'
        '${_workingValue.substring(12, 14)}';
    return _removeSpecialChar(_workingValue);
  }

  static String _maskCpf(String value) {
    var _workingValue = _onlyNumbers(value).padRight(14, _specialChar);
    _workingValue = '${_workingValue.substring(0, 3)}.'
        '${_workingValue.substring(3, 6)}.'
        '${_workingValue.substring(6, 9)}/'
        '${_workingValue.substring(9, 11)}';
    return _removeSpecialChar(_workingValue);
  }

  static String _removeSpecialChar(String value) {
    var _workingValue = value;
    if (_workingValue.contains(_specialChar)) {
      _workingValue.substring(0, _workingValue.indexOf(_specialChar));
    }
    return _workingValue;
  }
}

class _CnpjCpfFormatter extends TextInputFormatter {
  final bool cnpj;
  final bool cpf;

  _CnpjCpfFormatter({
    this.cnpj = true,
    this.cpf = true,
  });
  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue,
    TextEditingValue newValue,
  ) {
    var _workingValue = CnpjCpf._onlyNumbers(newValue.text);
    if (CnpjCpf.isCpf(_workingValue) && cpf) {
      _workingValue = CnpjCpf._maskCpf(_workingValue);
    }
    if (CnpjCpf.isCnpj(_workingValue) && cnpj) {
      _workingValue = CnpjCpf._maskCnpj(_workingValue);
    }
    return TextEditingValue(
      text: _workingValue,
      composing: TextRange(
          end: _workingValue.length - 1, start: _workingValue.length - 1),
      selection: TextSelection(
        baseOffset: _workingValue.length,
        extentOffset: _workingValue.length,
      ),
    );
  }
}
